#!/bin/bash

# Script bash che esegue la ricerca lineare di un 
# elemento in un array di numeri interi.
# Nota che in questo script sono introdotte le 
# variabili locali copia e riferimento.

printarray()
{
  local -n w=${1:-} # w è un riferimento ai parametri da $1 $2 ...,
                    # cioè all'array passato come argomento (v);
                    # il riferimento evita la copia dell'array (che è
                    # un'operazione onerosa) consentendo, inoltre,
                    # la modifica dell'array riferito (v)
  
  echo "array     : ${v[@]}"
  echo "dimensione: ${#v[@]}"
}

riclin()
{
  # gestione dei parametri
  local a=$1        # a è una copia di $1
  local -n w=${2:-} # w è un riferimento ai parametri da $2 in poi,
                    # cioè all'array passato come secondo argomento (v);
                    # il riferimento evita la copia dell'array (che è
                    # un'operazione onerosa) consentendo, inoltre,
                    # la modifica dell'array riferito (v)

  # visita degli elementi dell'array w (riferimento a v) 
  # e confronto con a (copia di $1)
  for (( i = 0 ; i < ${#w[@]} ; i++ )) ; do
    if [[ ${w[i]} == $a ]] ; then
      # in caso di successo viene tornato l'indice dell'elemento
      return $i
    fi
  done
  # in caso di insuccesso viene ritornato ${#w[@]}, cioè 
  # la dimensione di w, che è un indice non valido per l'array
  return ${#w[@]} 
}

printresult()
{
  local i=$1      # copia di $1
  local isup=$2   # copia di $2

  if [[ $i < $isup ]] ; then
    echo "Elemento trovato all'indice $i" 
  else
    echo "Elemento non trovato\n"
  fi
}

clear                        # cancella lo schermo
echo "$0 $@"                 # stampa nome script ($0) e parametri ($@)

v=(2 4 22 55 7 9 3 77)       # definisce un array
printarray "v"               # invoca una funzione che stampa l'array

riclin "$1" "v"              # invoca la funzione di ricerca lineare
i=$?                         # salva il valore tornato in i

sizeofv=${#v[@]}             # salva il primo indice non valido in sizeofv
printresult "$i" "$sizeofv"  # invoca la funzione di stampa risultato
